package biggests.gui;

import javax.swing.ButtonGroup;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import com.lowagie.text.pdf.DefaultFontMapper;
import java.awt.event.MouseAdapter;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import java.awt.event.MouseEvent;
import org.apache.batik.dom.GenericDOMImplementation;
import org.jfree.chart.plot.XYPlot;
import java.awt.event.ItemListener;
import java.io.Writer;
import java.io.File;
import javax.swing.JLabel;
import java.io.OutputStreamWriter;
import org.w3c.dom.DOMImplementation;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import java.awt.Component;
import org.w3c.dom.Document;
import javax.swing.JDialog;
import org.jfree.chart.ChartUtilities;
import javax.swing.JTextField;
import org.apache.batik.svggen.SVGGraphics2D;
import java.awt.Frame;
import java.io.FileOutputStream;
import java.awt.event.ItemEvent;
import biggests.utils.FileFilterExtension;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.io.BufferedOutputStream;
import java.io.OutputStream;
import com.lowagie.text.*;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
import org.jfree.chart.JFreeChart;
import java.awt.Graphics2D;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * <p>Title: Save Chart Image Dialog</p>
 *
 * <p>Description: Dialog suited for prompting the user to provide the
 *                 width, height and file format to export a chart as
 *                 an image.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonalves
 */
public class SaveChartImageDialog extends JDialog {
    /**
     * PNG document file extension.
     */
    public static final String PNG = ".png";

    /**
     * SVG document file extension.
     */
    public static final String SVG = ".svg";

    /**
     * PDF document file extension.
     */
    public static final String PDF = ".pdf";

    /**
     * The width of the image.
     */
    private int imageWidth;

    /**
     * The height of the image.
     */
    private int imageHeight;

    /**
     * Component(s) to export to image file(s).
     */
    private Component chartPanel;

    /**
     * Image size label.
     */
    private JLabel imageSizeLabel;

    /**
     * File path label.
     */
    private JLabel fileLabel;

    /**
     * File path input text field.
     */
    private JTextField fileTextField;

    /**
     * Browse file path button.
     */
    private JButton browseButton;

    /**
     * Displayed image size radio button.
     */
    private JRadioButton displayedSizeRadioButton;

    /**
     * Custom size radio button
     */
    private JRadioButton customSizeRadioButton;

    /**
     * Width label.
     */
    private JLabel widthLabel;

    /**
     * Width input text field.
     */
    private JTextField widthTextField;

    /**
     * Height label.
     */
    private JLabel heightLabel;

    /**
     * Height input text field.
     */
    private JTextField heightTextField;

    /**
     * Size button group.
     */
    private ButtonGroup sizeButtonGroup;

    /**
     * Save button.
     */
    private JButton saveButton;

    /**
     * Cancel button.
     */
    private JButton cancelButton;

    /**
     * Creates a save chart image dialog.
     *
     * @param owner <code>Frame</code> the owner of the dialog
     * @param chartPanel the <code>Component</code> to be
     *        exported to image file
     * @param title <code>String</code> the title of the dialog box
     * @param modal <code>boolean</code> <code>true</code> if dialog
     *        box should be modal, <code>false</code> otherwise
     * @param imageWidth int the width of the image
     * @param imageHeight int the height of the image
     */
    public SaveChartImageDialog(Frame owner, Component chartPanel,
                                String title, boolean modal,
                                int imageWidth, int imageHeight) {
        super(owner, title, modal);
        setSize(300, 300);
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.chartPanel = chartPanel;
        jbInit();
    }

    /**
     * Initializes GUI components.
     */
    public void jbInit() {
        JPanel contentPane = (JPanel)this.getContentPane();
        contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20,
                20));
        contentPane.setLayout(new SpringLayout());
        imageSizeLabel = new JLabel("Image size");
        displayedSizeRadioButton = new JRadioButton("as displayed");
        customSizeRadioButton = new JRadioButton("custom");
        customSizeRadioButton.setSelected(true);
        sizeButtonGroup = new ButtonGroup();
        displayedSizeRadioButton.addItemListener(new
                SaveChartImageDialog_displayedSizeRadioButton_itemAdapter(this));
        sizeButtonGroup.add(displayedSizeRadioButton);
        sizeButtonGroup.add(customSizeRadioButton);
        widthLabel = new JLabel("width");
        widthTextField = new JTextField(4);
        widthTextField.setText("" + imageWidth);
        heightLabel = new JLabel("height");
        heightTextField = new JTextField(4);
        heightTextField.setText("" + imageHeight);
        fileLabel = new JLabel("Filename");
        fileTextField = new JTextField(8);
        browseButton = new JButton("Browse...");
        browseButton.addActionListener(new
                SaveChartImageDialog_browseButton_actionAdapter(this));
        saveButton = new JButton("Save");
        cancelButton = new JButton("Cancel");
        saveButton.addActionListener(new
                SaveChartImageDialog_saveButton_actionAdapter(this));
        cancelButton.addActionListener(new
                SaveChartImageDialog_cancelButton_actionAdapter(this));

        SpringLayout layout = (SpringLayout) contentPane.getLayout();
        contentPane.add(imageSizeLabel);
        contentPane.add(displayedSizeRadioButton);
        layout.putConstraint(SpringLayout.WEST, displayedSizeRadioButton,
                             20,
                             SpringLayout.WEST, imageSizeLabel);

        layout.putConstraint(SpringLayout.NORTH, displayedSizeRadioButton,
                             5,
                             SpringLayout.SOUTH, imageSizeLabel);
        contentPane.add(customSizeRadioButton);
        layout.putConstraint(SpringLayout.WEST, customSizeRadioButton, 20,
                             SpringLayout.WEST, imageSizeLabel);
        layout.putConstraint(SpringLayout.NORTH, customSizeRadioButton, 5,
                             SpringLayout.SOUTH, displayedSizeRadioButton);
        contentPane.add(widthLabel);
        layout.putConstraint(SpringLayout.WEST, widthLabel, 30,
                             SpringLayout.WEST, imageSizeLabel);
        layout.putConstraint(SpringLayout.NORTH, widthLabel, 5,
                             SpringLayout.SOUTH, customSizeRadioButton);
        contentPane.add(widthTextField);
        layout.putConstraint(SpringLayout.WEST, widthTextField, 5,
                             SpringLayout.EAST, widthLabel);
        layout.putConstraint(SpringLayout.SOUTH, widthTextField, 0,
                             SpringLayout.SOUTH, widthLabel);
        contentPane.add(heightLabel);
        layout.putConstraint(SpringLayout.WEST, heightLabel, 10,
                             SpringLayout.EAST, widthTextField);
        layout.putConstraint(SpringLayout.SOUTH, heightLabel, 0,
                             SpringLayout.SOUTH, widthLabel);
        contentPane.add(heightTextField);
        layout.putConstraint(SpringLayout.WEST, heightTextField, 5,
                             SpringLayout.EAST, heightLabel);
        layout.putConstraint(SpringLayout.SOUTH, heightTextField, 0,
                             SpringLayout.SOUTH, heightLabel);

        contentPane.add(fileLabel);
        layout.putConstraint(SpringLayout.WEST, fileLabel,
                             0,
                             SpringLayout.WEST, imageSizeLabel);
        layout.putConstraint(SpringLayout.NORTH, fileLabel,
                             30,
                             SpringLayout.SOUTH, widthLabel);

        contentPane.add(fileTextField);
        layout.putConstraint(SpringLayout.WEST, fileTextField, 10,
                             SpringLayout.EAST, fileLabel);
        layout.putConstraint(SpringLayout.SOUTH, fileTextField, 0,
                             SpringLayout.SOUTH, fileLabel);

        contentPane.add(browseButton);
        layout.putConstraint(SpringLayout.WEST, browseButton, 10,
                             SpringLayout.EAST, fileTextField);
        layout.putConstraint(SpringLayout.SOUTH, browseButton, 0,
                             SpringLayout.SOUTH, fileTextField);

        contentPane.add(saveButton);
        layout.putConstraint(SpringLayout.WEST, saveButton, 50,
                             SpringLayout.WEST, fileLabel);
        layout.putConstraint(SpringLayout.SOUTH, saveButton, 50,
                             SpringLayout.SOUTH, fileLabel);
        contentPane.add(cancelButton);
        layout.putConstraint(SpringLayout.WEST, cancelButton, 50,
                             SpringLayout.EAST, saveButton);
        layout.putConstraint(SpringLayout.SOUTH, cancelButton, 0,
                             SpringLayout.SOUTH, saveButton);
    }

    /**
     * Handles the event fired when the user clicks save button.
     *
     * @param e the <code>ActionEvent</code> fired when the user
     *        clicks save button
     */
    public void saveButton_actionPerformed(ActionEvent e) {
        if (customSizeRadioButton.isSelected()) {
            try {
                readWidthInput();
                readHeightInput();
            }
            catch (Exception ex) {
                return;
            }
        }

        /**
         * user hit approve option; get the specified path for the file
         */
        String filePath = fileTextField.getText();
        if (filePath == null || filePath.equals("")){
            JOptionPane.showMessageDialog(this.getOwner(),
                                          "No filename." +
                                          "\nYou must provide a filename.",
                                          "Filename error",
                                          JOptionPane.ERROR_MESSAGE);
            return;
        }
        String extension = filePath.substring(filePath.length()-4,
                                              filePath.length());

        try {
            File newFile = new File(filePath);
            if (newFile.exists()) {
                if (newFile.isDirectory()) {
                    JOptionPane.showMessageDialog(this.getOwner(),
                                                  "Unable to save image." +
                                                  "\nThe specified filename " +
                                                  "corresponds to a directory.",
                                                  "Filename error",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (newFile.isFile()) {
                    int opt = JOptionPane.showOptionDialog(this.
                            getOwner(),
                            "File already exists." +
                            "\nOverwrite?",
                            "File exists",
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE,
                            null, null, JOptionPane.YES_OPTION);
                    if (opt == JOptionPane.NO_OPTION) {
                        /**
                         * the user does not want to overwrite the file
                         */
                        return;
                    }
                }
            }
            if (extension.equals(PNG)) {
                ChartUtilities.saveChartAsPNG(newFile,
                                              ((PanelExpressionChart)chartPanel).getChart(),
                                              imageWidth, imageHeight);
            } else
            if (extension.equals(SVG)){
                saveChartAsSVG(
                        newFile,
                        ((PanelExpressionChart)chartPanel).getChart(),
                        imageWidth, imageHeight);
            }
            else
            if (extension.equals(PDF)){
                saveChartAsPDF(newFile,
                               ((PanelExpressionChart)chartPanel).getChart(),
                               imageWidth, imageHeight,
                               new DefaultFontMapper());
            }
            else {
                JOptionPane.showMessageDialog(this.getOwner(),
                                              "Invalid file extension." +
                                              "\nYour filename must contain " +
                                              "a valid file extension.",
                                              "Filename error",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }
        }
        catch (Exception ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                                          "Unable to save image file",
                                          "Error saving image file",
                                          JOptionPane.INFORMATION_MESSAGE);
        }
        finally {
            this.dispose();
        }
    }

    /**
     * Saves a chart as a SVG document.
     *
     * @param file the <code>File</code>
     * @param chart the <code>JFreeChart</code>
     * @param width int the chart width
     * @param height int the chart height
     * @throws IOException
     */
    public static void saveChartAsSVG(File file, JFreeChart chart,
                                      int width, int height) throws IOException {
        Writer out = new OutputStreamWriter(
            new FileOutputStream(file), "UTF-8");
        writeChartAsSVG(out, chart, width, height);
        out.close();
    }

    /**
     * Writes a chart to an output stream in SVG document format.
     *
     * @param out the <code>Writer</code> output stream
     * @param chart the <code>JFreeChart</code>
     * @param width int the width
     * @param height int the height
     * @throws IOException
     */
    public static void writeChartAsSVG(Writer out,
                                       JFreeChart chart,
                                       int width,
                                       int height) throws IOException {
        // THE FOLLOWING CODE IS BASED ON AN EXAMPLE OF THE BATIK DOCUMENTATION...
        // Get a DOMImplementation
        DOMImplementation domImpl
                = GenericDOMImplementation.getDOMImplementation();
        // Create an instance of org.w3c.dom.Document
        Document document = domImpl.createDocument(null, "svg", null);
        // Create an instance of the SVG Generator
        SVGGraphics2D svgGenerator = new SVGGraphics2D(document);
        // set the precision to avoid a null pointer exception in Batik 1.5
        svgGenerator.getGeneratorContext().setPrecision(6);
        // Ask the chart to render into the SVG Graphics2D implementation
        chart.draw(
                svgGenerator,
                new Rectangle2D.Double(0, 0, width,
                height),
                null);
        // Finally, stream out SVG to a file using UTF-8 character to
        // byte encoding
        boolean useCSS = true;
        svgGenerator.stream(out, useCSS);
    }

    /**
     * Saves a chart to a PDF file.
     *
     * @param file the <code>File</code>
     * @param chart the <code>JFreeChart</code>
     * @param width int the chart width
     * @param height int the chart height
     * @param mapper FontMapper
     * @throws IOException
     */
    public static void saveChartAsPDF(File file,
                                      JFreeChart chart,
                                      int width,
                                      int height,
                                      FontMapper mapper) throws IOException {
        OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
        writeChartAsPDF(out, chart, width, height, mapper);
        out.close();
    }

    /**
     * Writes a chart to an output stream in PDF format.
     *
     * @param out the <code>OutputStream</code>
     * @param chart the <code>JFreeChart</code>
     * @param width int the width
     * @param height int the height
     * @param mapper FontMapper
     * @throws IOException
     */
    public static void writeChartAsPDF(OutputStream out,
                                       JFreeChart chart,
                                       int width,
                                       int height,
                                       FontMapper mapper) throws IOException {
        Rectangle pagesize = new Rectangle(width, height);
        com.lowagie.text.Document document =
            new com.lowagie.text.Document(pagesize, 50, 50, 50, 50);
        try {
            PdfWriter writer = PdfWriter.getInstance(document, out);
            document.addAuthor("BiGGEsTS");
            document.addSubject("Expression Chart");
            document.open();
            PdfContentByte cb = writer.getDirectContent();
            PdfTemplate tp = cb.createTemplate(width, height);
            Graphics2D g2 = tp.createGraphics(width, height, mapper);
            Rectangle2D r2D = new Rectangle2D.Double(0, 0, width, height);
            chart.draw(g2, r2D);
            g2.dispose();
            cb.addTemplate(tp, 0, 0);
        } catch (DocumentException de) {
            System.err.println(de.getMessage());
        }
        document.close();
    }

    /**
     * Handles the event fired when the user clicks cancel button.
     *
     * @param e the <code>ActionEvent</code> fired when the user
     *        clicks save button
     */
    public void cancelButton_actionPerformed(ActionEvent e) {
        this.dispose();
    }

    /**
     * Reads the width input from the corresponding textfield.
     *
     * @throws Exception
     */
    public void readWidthInput() throws Exception {
        int width;
        try {
            width = Integer.parseInt(widthTextField.getText());
        }
        catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Width must be an integer value.",
                    "Width must be integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }

        if (width <= 0) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Width must be a positive integer value.",
                    "Width must be a positive integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }
        imageWidth = width;
    }

    /**
     * Reads the height input from the corresponding text field.
     *
     * @throws Exception
     */
    public void readHeightInput() throws Exception {
        int height;
        try {
            height = Integer.parseInt(heightTextField.getText());
        }
        catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Height must be an integer value.",
                    "Height must be integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }

        if (height <= 0) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Height must be a positive integer value.",
                    "Height must be a positive integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }
        imageHeight = height;
    }

    /**
     * Handles the event fired when the state of the displayed
     * size radio button changes.
     *
     * @param itemEvent the <code>ItemEvent</code> fired when the state
     *        of the displayed size radio button changes
     */
    public void displayedSizeRadioButton_itemStateChanged(ItemEvent itemEvent) {
        boolean enable = !displayedSizeRadioButton.isSelected();
        widthLabel.setEnabled(enable);
        widthTextField.setEnabled(enable);
        heightLabel.setEnabled(enable);
        heightTextField.setEnabled(enable);
    }

    public void browseButton_actionPerformed(ActionEvent e) {
        JFileChooser pathFileChooser = new JFileChooser();
        try {
            /**
             * initialize the file chooser with the intended file filters
             * (one for png files, and another for svg files)
             */
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(".svg"));
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(".pdf"));
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(
                            ".png"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        /**
         * turn off accept all file filter
         */
        pathFileChooser.setAcceptAllFileFilterUsed(false);

        /**
         * show the save dialog to the user
         */
        int option = pathFileChooser.showSaveDialog(this);

        if (option == JFileChooser.APPROVE_OPTION) {
            /**
             * user hit approve option; get the specified path for the file
             */
            String newPath = pathFileChooser.getSelectedFile().getPath();
            newPath = ((FileFilterExtension) pathFileChooser.getFileFilter()).
                      checkAndAddFileExtension(newPath);
            fileTextField.setText(newPath);
        }
    }
}


class SaveChartImageDialog_browseButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_browseButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.browseButton_actionPerformed(e);
    }
}


class SaveChartImageDialog_displayedSizeRadioButton_itemAdapter implements
        ItemListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_displayedSizeRadioButton_itemAdapter(
            SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void itemStateChanged(ItemEvent itemEvent) {
        adaptee.displayedSizeRadioButton_itemStateChanged(itemEvent);
    }
}


class SaveChartImageDialog_cancelButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_cancelButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.cancelButton_actionPerformed(e);
    }
}


class SaveChartImageDialog_saveButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_saveButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.saveButton_actionPerformed(e);
    }
}
