package biggests.tables;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;

import biggests.utils.*;
import smadeira.biclustering.*;
import smadeira.ontologizer.*;
import biggests.gui.BiggestsFrame;

/**
 * <p>Title: Function Table for an Individual Bicluster</p>
 *
 * <p>Description: Contains data about function analysis performed on
 *                 a given bicluster.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */

public class TableFunctionIndividual extends JTable {
    /**
     * Default color for highly significant terms.
     */
    public static final Color HIGHLY_SIGNIFICANT_COLOR = new Color(89, 255, 89);

    /**
     * Default color for significant terms.
     */
    public static final Color SIGNIFICANT_COLOR = new Color(168, 255, 168);

    /**
     * Default color for even rows.
     */
    public static final Color EVEN_ROWS_COLOR = new Color(228, 238, 250);

    /**
     * Default color for odd rows.
     */
    public static final Color ODD_ROWS_COLOR = Color.WHITE;

    /**
     * P-value threshold to mark highly significant terms.
     */
    private float highlySignificantPValueThreshold;

    /**
     * P-value threshold to mark significant terms.
     */
    private float significantPValueThreshold;

    /**
     * Color to mark rows that contain highly significant terms.
     */
    private Color highlySignificantColor;

    /**
     * Color to mark rows that contain significant terms.
     */
    private Color significantColor;

    /**
     * Color to mark even rows (when they are not selected).
     */
    private Color evenRowsColor;

    /**
     * Color to mark odd rows (when they are not selected).
     */
    private Color oddRowsColor;

    /**
     * The genes names of the gene set (not the population set).
     */
    private String[] genesNames;

    /**
     * The GO Frontend with the GO graph of annotated terms.
     */
    private GOFrontEnd goFrontEnd;

    /**
     * This table's parent GUI component. It's used to specify the
     * location of the popup dialogs that display messages to the user.
     */
    private Component parent;

    /**
     * The abbreviated column names for the header of this table of functions.<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - #A. Population G.<br/>
     * 4th column - #A. Set G.<br/>
     * 5th column - P-Value<br/>
     * 6th column - Corrected P-Value<br/>
     */
    private static final String[] columnNames = {"GO Term ID",
                                                "GO Term Name",
                                                "#A. Population G.",
                                                "#A. Set G.",
                                                "P-Value",
                                                "Corrected P-Value"};

    /**
     * The column names for the header of this table of functions.<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Annotated Population Genes<br/>
     * 4th column - Annotated Set Genes<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected P-Value<br/>
     */
    private static final String[] completeColumnNames = {"GO Term ID",
            "GO Term Name",
            "Annotated Population Genes",
            "Annotated Set Genes",
            "Hypergeometric P-Value",
            "Bonferroni Corrected P-Value"};

    /**
     * <p>
     * Creates a new table of functions from the data of a bicluster object.
     * A table of functions for a bicluster contains the following
     * information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     * 4th column - Number of Genes in Set Annotated with the Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     * </p>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param bicluster the <code>Bicluster</code> from which the
     *                   function table is created
     * @param goProperties the <code>GeneSetResult</code> for the given
     *        <code>bicluster</code>
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param pValueThreshold the significancy threshold <code>float</code> p-value to
     *        color significant terms (GO terms are significant if their
     *        Bonferroni corrected hypergeometric p-value is below
     *        <code>pValueThreshold</code>
     * @param parent the GUI's parent <code>Component</code>
     * @throws InvalidNodeObjectTypeException when the type of the bicluster
     *                   object is not valid
     * @throws Exception if <code>bicluster</code> has no gene set result
     *                   with GO terms associated with it
     * @see #constructRowData(GeneSetResult goProperties, boolean onlyTerms)
     * @see #TableFunctionGroup(Object[][] rowData, Object[] columnNames))
     */
    /* --------------------------------------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Set Genes | P-Value | Corrected P-Value |<br/>
     * |------------------------------------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       | 0.0093..|        1.0        |<br/>
     * | GO:0051704 | interaction... |        16        |         1       | 0.0831..|        1.0        |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * --------------------------------------------------------------------------------------------------<br/>
     */
    public TableFunctionIndividual(Bicluster bicluster, GeneSetResult goProperties, GOFrontEnd go,
                                   float pValueThreshold, Component parent) throws
            InvalidNodeObjectTypeException, Exception {
        this(constructRowData(goProperties, false),
             columnNames, go, pValueThreshold);
        genesNames = bicluster.getGenesNames();
        this.parent = parent;
    }

    /**
     * Creates a new function table from the GO terms infotmation
     * in <code>matrix</code>. It contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>
     * <tr><td>GO:0007275<td> development<td>37
     * <tr><td>GO:0051704<td>interaction...<td>16
     * <tr><td>...<td>...<td>...
     * </table>
     *
     * @param matrix the <code>IMatrix</code> which contains the GO
     *        terms to present in the table
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @throws Exception if GO terms information does not exist
     * @see #TableFunctionIndividualTable(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(GeneSetResult goProperties, boolean onlyTerms)
     * @see #getActualColumnNames(boolean onlyTerms)
     */
    public TableFunctionIndividual(IMatrix matrix, GOFrontEnd go) throws
            Exception {
        this(constructRowData(matrix.getGeneOntologyProperties(), true),
             getActualColumnNames(true), go);
        genesNames = matrix.getGenesNames();
    }

    /**
     * Creates a new function table from the given GO terms.
     * A table of functions for a bicluster contains the following
     * information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     * 4th column - Number of Genes in Set Annotated with the Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param goProperties the <code>GeneSetResult</code> which contains the
     *                   GO terms annotated for the genes
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param genesNames <code>String[]</code> the names of the genes in gene set
     * @param pValueThreshold the significancy threshold <code>float</code> p-value to
     *        color significant terms (GO terms are significant if their
     *        Bonferroni corrected hypergeometric p-value is below
     *        <code>pValueThreshold</code>
     * @throws InvalidNodeObjectTypeException when the type of the bicluster
     *                   object is not valid
     * @throws Exception if <code>goProperties</code> is empty
     * @see #constructRowData(Bicluster bicluster)
     * @see #TableFunctionIndividual(Object[][] rowData, Object[] columnNames))
     */
    /* --------------------------------------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Set Genes | P-Value | Corrected P-Value |<br/>
     * |------------------------------------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       | 0.0093..|        1.0        |<br/>
     * | GO:0051704 | interaction... |        16        |         1       | 0.0831..|        1.0        |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * --------------------------------------------------------------------------------------------------<br/>
     */
    public TableFunctionIndividual(GeneSetResult goProperties, GOFrontEnd go,
                                   String[] genesNames, float pValueThreshold) throws
            Exception {
        this(constructRowData(goProperties, false), columnNames, go,
             pValueThreshold);
        this.genesNames = genesNames;
    }

    /**
     * Creates a new function table from the given GO terms.
     * It contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>
     * <tr><td>GO:0007275<td> development<td>37
     * <tr><td>GO:0051704<td>interaction...<td>16
     * <tr><td>...<td>...<td>...
     * </table>
     *
     * @param goProperties the <code>GeneSetResult</code> which contains the
     *                   GO terms annotated for the genes
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param genesNames <code>String[]</code> the names of the genes in gene set
     * @throws Exception if <code>goProperties</code> is empty
     * @see #constructRowData(GeneSetResult goProperties, boolean onlyTerms)
     * @see #getActualColumnNames(boolean onlyTerms)
     * @see #TableFunctionIndividual(Object[][] rowData, Object[] columnNames))
     */
    /* --------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Set Genes |<br/>
     * |------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       |<br/>
     * | GO:0051704 | interaction... |        16        |         1       |<br/>
     * | .......... | .............. | ................ | ............... |<br/>
     * | .......... | .............. | ................ | ............... |<br/>
     * --------------------------------------------------------------------<br/>
     */
    public TableFunctionIndividual(GeneSetResult goProperties, GOFrontEnd go,
                                   String[] genesNames) throws Exception {
        this(constructRowData(goProperties, true), getActualColumnNames(true),
             go);
        this.genesNames = genesNames;
    }

    /**
     * Creates a new table of functions with terms and p-values data.
     * It contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     * 4th column - Number of Genes in Bicluster Annotated with the Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     * GO terms with hypergeometric p-value lower than <code>pValueThreshold</code>
     * are colored differently.
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param rowData the <code>Object[][]</code> row data (for all table
     *                cells except the ones from the table header)
     * @param columnNames the <code>Object[]</code> column names (for
     *                the table header)
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param pValueThreshold the significancy threshold <code>float</code> p-value to
     *        color significant terms (GO terms are significant if their
     *        Bonferroni corrected hypergeometric p-value is below
     *        <code>pValueThreshold</code>
     * @see #TableFunctionIndividual(TableSorterFunctionIndividual
     *      tableSorterModel, float pValueThreshold)
     * @see applicationgui.utils.TableSorter#TableSorter(TableModel tableModel)
     * @see javax.swing.table.DefaultTableModel#DefaultTableModel(Object[][]
     *      rowData, Object[] columnNames)
     * @throws Exception
     */
    /* --------------------------------------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Bic Genes | P-Value | Corrected P-Value |<br/>
     * |------------------------------------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       | 0.0093..|        1.0        |<br/>
     * | GO:0051704 | interaction... |        16        |         1       | 0.0831..|        1.0        |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * --------------------------------------------------------------------------------------------------<br/>
     */
    protected TableFunctionIndividual(
            Object[][] rowData, Object[] columnNames, GOFrontEnd go,
            float pValueThreshold) throws Exception {
        /**
         * create a new JTable with a table sorter model, which incorporates
         * the specified row data and column names
         */
        this(new TableSorterFunctionIndividual(new DefaultTableModel(rowData,
                columnNames)), go, pValueThreshold);
    }

    /**
     * Creates a new table of functions with GO terms and p-values data.
     * It contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     * 4th column - Number of Genes in Bicluster Annotated with the Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     * GO terms with hypergeometric p-value lower than <code>pValueThreshold</code>
     * are colored differently.
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param rowData the <code>Object[][]</code> row data (for all table
     *                cells except the ones from the table header)
     * @param columnNames the <code>Object[]</code> column names (for
     *                the table header)
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param highlySignificantPValueThreshold <code>float</code> the maximum
     *        Bonferroni corrected hypergeometric p-value to color highly
     *        significant GO terms; GO terms are highly significant if
     *        their Bonferroni corrected hypergeometric p-value is in
     *        [0, highlySignificantPValueThreshold[
     * @param significantPValueThreshold <code>float</code> the maximum
     *        p-value to color significant GO terms; GO terms are significant
     *        if their Bonferroni corrected hypergeometric p-value is in
     *        [highlySignificantPValueThreshold, significantPValueThreshold[
     * @see #TableFunctionIndividual(TableSorterFunctionIndividual
     *      tableSorterModel, float highlySignificantPValueThreshold,
     *      float significantPValueThreshold)
     * @see applicationgui.utils.TableSorter#TableSorter(TableModel tableModel)
     * @see javax.swing.table.DefaultTableModel#DefaultTableModel(Object[][]
     *      rowData, Object[] columnNames)
     * @throws Exception
     */
    /* --------------------------------------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Bic Genes | P-Value | Corrected P-Value |<br/>
     * |------------------------------------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       | 0.0093..|        1.0        |<br/>
     * | GO:0051704 | interaction... |        16        |         1       | 0.0831..|        1.0        |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * --------------------------------------------------------------------------------------------------<br/>
     */
    protected TableFunctionIndividual(Object[][] rowData,
                                      Object[] columnNames, GOFrontEnd go,
                                      float highlySignificantPValueThreshold,
                                      float significantPValueThreshold) throws
            Exception {
        this(new TableSorterFunctionIndividual(
                new DefaultTableModel(rowData, columnNames)), go,
             highlySignificantPValueThreshold,
             significantPValueThreshold);
    }

    /**
     * Creates a new table of functions from the given <code>rowData</code>
     * and <code>columnNames</code>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with the Term<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes
     * <tr><td>GO:0007275<td> development<td>37
     * <tr><td>GO:0051704<td>interaction...<td>16
     * <tr><td>...<td>...<td>...
     * </table>
     *
     * @param rowData Object[][]
     * @param columnNames Object[]
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @see applicationgui.utils.TableSorter#TableSorter(TableModel tableModel)
     * @see javax.swing.table.DefaultTableModel#DefaultTableModel(Object[][]
     *      rowData, Object[] columnNames)
     */
    /* --------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Set Genes |<br/>
     * |------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       |<br/>
     * | GO:0051704 | interaction... |        16        |         1       |<br/>
     * | .......... | .............. | ................ | ............... |<br/>
     * | .......... | .............. | ................ | ............... |<br/>
     * --------------------------------------------------------------------<br/>
     */
    protected TableFunctionIndividual(Object[][] rowData, Object[] columnNames,
                                      GOFrontEnd go) {
        this(new TableSorterFunctionIndividual(new DefaultTableModel(rowData,
                columnNames)), go);
    }

    /**
     * Creates a new table of functions from the given
     * <code>tableSorterModel</code>.
     * The table contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with this Term<br/>
     * 4th column - Number of Genes in Set Annotated with this Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param tableSorterModel the <code>TableSorterFunctionIndividual</code>
     *        model for the table
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param pValueThreshold the significancy threshold <code>float</code>
     *        p-value to color significant terms (GO terms are significant
     *        if their Bonferroni corrected hypergeometric p-value is below
     *        <code>pValueThreshold</code>
     * @see javax.swing.JTable#JTable(TableModel dm)
     * @throws Exception
     */
    /* --------------------------------------------------------------------------------------------------<br/>
     * | GO Term ID | GO Term Name   | #Ann. Pop. Genes | #Ann. Bic Genes | P-Value | Corrected P-Value |<br/>
     * |------------------------------------------------------------------------------------------------|<br/>
     * | GO:0007275 | development    |        37        |         2       | 0.0093..|        1.0        |<br/>
     * | GO:0051704 | interaction... |        16        |         1       | 0.0831..|        1.0        |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * | .......... | .............. | ................ | ............... | ........| ................. |<br/>
     * --------------------------------------------------------------------------------------------------<br/>
     */
    protected TableFunctionIndividual(
            TableSorterFunctionIndividual tableSorterModel, GOFrontEnd go,
            float pValueThreshold) throws Exception {
        this(tableSorterModel, go, pValueThreshold, pValueThreshold);
    }

    /**
     * Creates a new table of functions for the given <code>tableSorterModel</code>.
     * The table contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with this Term<br/>
     * 4th column - Number of Genes in Set Annotated with this Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes<th>#Ann. Set Genes<th>P-Value<th> Corrected P.Value
     * <tr><td>GO:0007275<td> development<td>37<td>2<td>0.0093..<td>1.0
     * <tr><td>GO:0051704<td>interaction...<td>16<td>1<td>0.0831..<td>1.0
     * <tr><td>...<td>...<td>...<td>...<td>...<td>...
     * </table>
     *
     * @param tableSorterModel the <code>TableSorterFunctionIndividual</code>
     *        model for the table
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     * @param highlySignificantPValueThreshold <code>float</code> the maximum
     *        Bonferroni corrected hypergeometric p-value to color highly
     *        significant GO terms; GO terms are highly significant if
     *        their Bonferroni corrected hypergeometric p-value is in
     *        [0, highlySignificantPValueThreshold[
     * @param significantPValueThreshold <code>float</code> the maximum
     *        p-value to color significant GO terms; GO terms are significant
     *        if their Bonferroni corrected hypergeometric p-value is in
     *        [highlySignificantPValueThreshold, significantPValueThreshold[
     * @throws Exception
     */
    protected TableFunctionIndividual(
            TableSorterFunctionIndividual tableSorterModel, GOFrontEnd go,
            float highlySignificantPValueThreshold,
            float significantPValueThreshold) throws Exception {
        /**
         * create a new JTable object with a table sorter model derived from the
         * given table model
         */
        super(tableSorterModel);
        this.goFrontEnd = go;

        this.addMouseListener(new TableFunctionIndividual_this_mouseAdapter(this));

        if (highlySignificantPValueThreshold > significantPValueThreshold) {
            throw new Exception("P-value threshold for highly significant " +
                                "gene ontology terms must be equal or " +
                                "higher than the p-value threshold for " +
                                "significant ones.");
        }

        /**
         * set table header of this TableValues as the table header of the
         * created table sorter model
         */
        ((TableSorterFunctionIndividual)this.getModel()).setTableHeader(this.
                getTableHeader());

        this.getTableHeader().setReorderingAllowed(false);

        this.setDefaultRenderer(Float.class, new FloatCellRenderer());

        /**
         * set preferred size for all table's cells
         */
        Enumeration<TableColumn> e = this.getColumnModel().getColumns();
        while (e.hasMoreElements()) {
            e.nextElement().setPreferredWidth(100);
        }

        this.getColumnModel().getColumn(1).setPreferredWidth(300);
        if (this.getColumnModel().getColumnCount() > 2) {
            this.getColumnModel().getColumn(5).setPreferredWidth(120);
        }

        /**
         * set header font to bold style
         */
        JTableHeader tableHeader = this.getTableHeader();
        Font headerFont = tableHeader.getFont();
        tableHeader.setFont(new Font(headerFont.getName(), Font.BOLD,
                                     headerFont.getSize()));

        this.highlySignificantPValueThreshold =
                highlySignificantPValueThreshold;
        this.significantPValueThreshold = significantPValueThreshold;
        this.highlySignificantColor = this.HIGHLY_SIGNIFICANT_COLOR;
        this.significantColor = this.SIGNIFICANT_COLOR;
        this.evenRowsColor = this.EVEN_ROWS_COLOR;
        this.oddRowsColor = this.ODD_ROWS_COLOR;
    }

    /**
     * Creates a new table of functions for the given
     * <code>tableSorterModel</code>.
     * The table contains the following information:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with this Term<br/>
     *
     * <table border="1">
     * <tr><th>GO Term ID<th>GO Term Name<th>#Ann. Pop. Genes
     * <tr><td>GO:0007275<td> development<td>37
     * <tr><td>GO:0051704<td>interaction...<td>16
     * <tr><td>...<td>...<td>...
     * </table>
     *
     * @param tableSorterModel <code>TableSorterFunctionIndividual</code>
     *        the model to build the table from
     * @param go GOFrontEnd the GO frontend to the gene ontology for this
     *        table's population of genes
     */
    protected TableFunctionIndividual(
            TableSorterFunctionIndividual tableSorterModel, GOFrontEnd go) {
        /**
         * create a new JTable object with a table sorter model derived from the
         * given table model
         */
        super(tableSorterModel);
        this.goFrontEnd = go;

        this.addMouseListener(new TableFunctionIndividual_this_mouseAdapter(this));

        /**
         * set table header of this TableValues as the table header of the
         * created table sorter model
         */
        ((TableSorterFunctionIndividual)this.getModel()).setTableHeader(this.
                getTableHeader());

        this.getTableHeader().setReorderingAllowed(false);

        /**
         * set preferred size for all table's cells
         */
        Enumeration<TableColumn> e = this.getColumnModel().getColumns();
        e.nextElement().setPreferredWidth(50);
        e.nextElement().setPreferredWidth(400);
        while (e.hasMoreElements()) {
            e.nextElement().setPreferredWidth(50);
        }

        /**
         * set header font to bold style
         */
        JTableHeader tableHeader = this.getTableHeader();
        Font headerFont = tableHeader.getFont();
        tableHeader.setFont(new Font(headerFont.getName(), Font.BOLD,
                                     headerFont.getSize()));

        this.evenRowsColor = this.EVEN_ROWS_COLOR;
        this.oddRowsColor = this.ODD_ROWS_COLOR;
    }

    /**
     * Overrides <code>JTable</code>'s <code>prepareRenderer</code> method.
     * Prepares the renderer by querying the data model for the
     * value and selection state
     * of the cell at <code>row</code>, <code>column</code>.
     * Returns the component (may be a <code>Component</code>
     * or a <code>JComponent</code>) under the event location.
     *
     * @param renderer  the <code>TableCellRenderer</code> to prepare
     * @param row       the row of the cell to render, where 0 is the first row
     * @param column    the column of the cell to render,
     *			where 0 is the first column
     * @return          the <code>Component</code> under the event location
     * @see javax.swing.JTable#prepareRenderer(TableCellRenderer renderer, int row, int column)
     */
    public Component prepareRenderer(TableCellRenderer renderer, int row,
                                     int column) {
        /**
         * get the component prepared from JTable's prepareRenderer method
         */
        Component c = super.prepareRenderer(renderer, row, column);

        if (this.getColumnModel().getColumnCount() != 3) {
            if (highlySignificantPValueThreshold == significantPValueThreshold) {
                if ((row % 2 == 0) && (!isCellSelected(row, column))) {
                    /**
                     * cell is in row numbered even and is not selected
                     */
                    float value = ((Float)this.getValueAt(row, 5)).floatValue();
                    if (value <= this.highlySignificantPValueThreshold) {
                        /**
                         * signal a row which corresponds to a GO Term with
                         * a significant corrected p-value
                         */
                        c.setBackground(this.highlySignificantColor);
                    } else {
                        /**
                         * cell is in an odd row, does not correspond to
                         * a significant term and it is not selected
                         */
                        c.setBackground(this.evenRowsColor);
                    }
                } else
                if (!isCellSelected(row, column)) {
                    /**
                     * cell is in row numbered odd and is not selected
                     */
                    float value = ((Float)this.getValueAt(row, 5)).floatValue();
                    if (value <= this.highlySignificantPValueThreshold) {
                        /**
                         * row is odd, corresponds to a GO Term with
                         * significant corrected p-value and it is not selected
                         */
                        c.setBackground(this.highlySignificantColor);
                    } else {
                        /**
                         * cell is in a row numbered odd, does not corresponde to
                         * a GO Term width significant p-value and is not selected
                         */
                        c.setBackground(this.oddRowsColor);
                    }
                }
            } else {
                if ((row % 2 == 0) && (!isCellSelected(row, column))) {
                    /**
                     * cell is in row numbered even and is not selected
                     */
                    float value = ((Float)this.getValueAt(row, 5)).floatValue();
                    if (value <= this.highlySignificantPValueThreshold) {
                        /**
                         * row is even, corresponds to a GO Term with
                         * a highly significant corrected p-value and
                         * it is not selected
                         */
                        c.setBackground(this.highlySignificantColor);
                    } else
                    if (value > this.highlySignificantPValueThreshold &&
                        value <= this.significantPValueThreshold) {
                        /**
                         * row is even, corresponds to a GO Term with
                         * a significant corrected p-value and it is not selected
                         */
                        c.setBackground(this.significantColor);
                    } else {
                        /**
                         * row is even and does not contain a significant term,
                         * nor it is selected
                         */
                        c.setBackground(this.evenRowsColor);
                    }
                } else
                if (!isCellSelected(row, column)) {
                    /**
                     * cell is in row numbered odd and is not selected
                     */
                    float value = ((Float)this.getValueAt(row, 5)).floatValue();
                    if (value <= this.highlySignificantPValueThreshold) {
                        /**
                         * row is odd, corresponds to a GO Term with
                         * a highly significant corrected p-value and
                         * is not selected
                         */
                        c.setBackground(this.highlySignificantColor);
                    } else
                    if (value > this.highlySignificantPValueThreshold &&
                        value <= this.significantPValueThreshold) {
                        /**
                         * row is odd, corresponds to a GO term with
                         * a significant corrected p-value and it is not selected
                         */
                        c.setBackground(this.significantColor);
                    } else {
                        /**
                         * cell is in a row numbered odd, does not corresponde to
                         * a GO Term width significant p-value and is not selected
                         */
                        c.setBackground(this.oddRowsColor);
                    }
                }
            }
        } else {
            if ((row % 2 == 0) && (!isCellSelected(row, column))) {
                /**
                 * cell is in row numbered even and is not selected
                 */
                c.setBackground(this.evenRowsColor);
            } else
            if (!isCellSelected(row, column)) {
                c.setBackground(this.oddRowsColor);
            }
        }
        return c;
    }

    /**
     * Overrides <code>JTable</code>'s <code>getToolTipText</code> method
     * in order to allow the renderer's tips to be used if they have text set.
     *
     * @param event MouseEvent the event that triggered the tool tip display
     * @return String the message set as tool tip for the pointed table cell
     * @see javax.swing.JTable#getToolTipText(MouseEvent event)
     */
    public String getToolTipText(MouseEvent event) {
        String tip = null;
        Point p = event.getPoint();
        int rowIndex = rowAtPoint(p);
        int colIndex = columnAtPoint(p);
        int realColumnIndex = convertColumnIndexToModel(colIndex);

        if (rowIndex < 0) {
            return "";
        }

        if (realColumnIndex == 0 || realColumnIndex == 1) {
            /**
             * cell is in the first column or second column, so it contains
             * the ID or the name of the GO Term, respectively;
             * set the tex tip for this cell, something like: "GO:0007264" or
             * "small GTPase mediated signal transduction"
             */
            tip = getValueAt(rowIndex, realColumnIndex).toString();
        } else
        if ((rowIndex >= 0) && (rowIndex < this.getRowCount()) &&
            (realColumnIndex > 0) && (realColumnIndex < this.getColumnCount())) {
            /**
             * cell is in a column that is not the first or the second one;
             * set the tip for this cell, something like:
             * "(GO:0007264, Corrected P-Value) = 0.02380094812477373"
             */
            tip = "(" + getValueAt(rowIndex, 0) +
                  ", " + this.completeColumnNames[realColumnIndex] +
                  ") = " + getValueAt(rowIndex, realColumnIndex);
        } else {
            tip = super.getToolTipText(event);
        }
        return tip;
    }

    /**
     * Overrides <code>JTable</code>'s <code>createDefaultTableHeader</code>
     * method.<br/>
     * Creates the default table header for a values' table, that is, a header
     * with specific text tool tips for this kind of table.
     *
     * @return an instance of <code>TableHeaderFunction</code>, i.e, a specific table
     *         header for a table of functions
     * @see javax.swing.JTable#columnModel
     * @see javax.swing.JTable#createDefaultTableHeader()
     * @see applicationgui.utils.TableHeader#TableHeader(TableColumnModel cm)
     */
    protected JTableHeader createDefaultTableHeader() {
        return new TableHeaderFunction(columnModel);
    }

    /**
     * Constructs the row data matrix with data retrieved from the bicluster
     * object.<br/>
     * <p>
     * The final structure of the row data is:<br/>
     * 1st column - GO Term ID<br/>
     * 2nd column - GO Term Name<br/>
     * 3rd column - Number of Genes in Population Annotated with this Term<br/>
     * 4th column - Number of Genes in Bicluster Annotated with this Term<br/>
     * 5th column - Hypergeometric P-Value<br/>
     * 6th column - Bonferroni Corrected Hypergeometric P-Value<br/>
     * </p>
     * <p>
     * <code>protected</code>: it's intended to be used only by the
     *                       <code>TableFunctionIndividual</code>'s constructor and
     *                       constructors of <code>TableFunctionIndividual</code>'s
     *                       subclasses.<br/>
     * <code>static</code>: because object <code>this</code> cannot be
     *                      referenced by the constructor before it is
     *                      actually created.
     * </p>
     *
     * @param goProperties the <code>GeneSetResult</code> which contains
     *                     the annotated GO terms
     * @param onlyTerms <code>boolean</code> <code>true</code> if the table
     *        should have only term information, no p-values;
     *        <code>false</code> otherwise
     * @return the <code>Object[][]</code> row data matrix, ready to be used by
     *         <code>TableFunctionIndividual</code>'s constructor
     * @throws Exception if gene set result is empty
     */
    protected static Object[][] constructRowData(GeneSetResult goProperties,
                                                 boolean onlyTerms) throws
            Exception {
        if (goProperties == null || goProperties.getNumberOfTerms() == 0) {
            throw new Exception("Gene set result is empty.");
        }

        if (onlyTerms) {
            Object[][] rowData = new Object[goProperties.getNumberOfTerms()][4];
            for (int i = 0; i < goProperties.getNumberOfTerms(); i++) {
                rowData[i][0] = goProperties.getTermID()[i];
                rowData[i][1] = goProperties.getTermName()[i];
                rowData[i][2] = Integer.valueOf(goProperties.
                                                getGenesInPopulationAnnotatedWithTerm()[
                                                i]);
            }

            return rowData;
        }

        Object[][] rowData = new Object[goProperties.getNumberOfTerms()][6];
        for (int i = 0; i < goProperties.getNumberOfTerms(); i++) {
            rowData[i][0] = goProperties.getTermID()[i];
            rowData[i][1] = goProperties.getTermName()[i];
            rowData[i][2] = Integer.valueOf(goProperties.
                                            getGenesInPopulationAnnotatedWithTerm()[
                                            i]);
            rowData[i][3] = Integer.valueOf(goProperties.
                                            getGenesInBiclusterAnnotatedWithTerm()[
                                            i]);
            rowData[i][4] = Float.valueOf((float) goProperties.getPValue()[
                                          i]);
            rowData[i][5] = Float.valueOf((float) goProperties.
                                          getCorrectedPValue()[i]);
        }
        return rowData;
    }

    /**
     * Returns the p-value threshold for highly significant gene ontology terms.
     *
     * @return float the p-value threshold for significant gene ontology terms
     */
    public float getHighlySignificantPValueThreshold() {
        return highlySignificantPValueThreshold;
    }

    /**
     * Returns the p-value threshold for significant gene ontology terms.
     *
     * @return float the p-value threshold for significant gene ontology terms
     */
    public float getSignificantPValueThreshold() {
        return significantPValueThreshold;
    }

    /**
     * Returns the <code>Color</code> used to mark highly significant terms.
     *
     * @return Color used to mark highly significant terms
     */
    public Color getHighlySignificantColor() {
        return highlySignificantColor;
    }

    /**
     * Returns the <code>Color</code> used to mark significant terms.
     *
     * @return Color used to mark significant terms
     */
    public Color getSignificantColor() {
        return significantColor;
    }

    /**
     * Returns the <code>Color</code> used to mark even rows that are not selected.
     *
     * @return Color used to mark even rows that are not selected
     */
    public Color getEvenRowsColor() {
        return evenRowsColor;
    }

    /**
     * Returns the <code>Color</code> used to mark odd rows that are not selected.
     *
     * @return Color used to mark odd rows that are not selected
     */
    public Color getOddRowsColor() {
        return oddRowsColor;
    }

    /**
     * Returns this table abbreviated columns names.
     *
     * @return the table <code>String[]</code> set of abbreviated columns names
     */
    public static String[] getColumnNames() {
        return columnNames;
    }

    /**
     * Returns this table complete columns names.
     *
     * @return the table <code>String[]</code> set of complete columns names
     */
    public static String[] getCompleteColumnNames() {
        return completeColumnNames;
    }

    /**
     * Returns the actual column names for the type of information
     * to be displayed. If <code>onlyTerms</code> is <code>true</code>,
     * columns names will be three (GO Term ID, GO Term Name, Number
     * of Genes in Population Annotated with the Term), otherwise will
     * be six (GO Term ID, GO Term Name, Number of Genes in Population
     * Annotated with the Term, Number of Genes in Set Annotated
     * with the Term, Hypergeometric P-Value, Bonferroni Corrected
     * Hypergeometric P-Value).
     *
     * @param onlyTerms <code>boolean</code> <code>true</code> if only
     * terms information should be displayed (no p-values information)
     * @return String[] the actual columns names
     */
    private static String[] getActualColumnNames(boolean onlyTerms) {
        if (onlyTerms) {
            String[] names = {columnNames[0], columnNames[1],
                             columnNames[2]};
            return names;
        }
        return columnNames;
    }

    /**
     * Returns the actual column names for this table. If this table
     * displays only terms information (no p-values), columns names
     * will be three (GO Term ID, GO Term Name, Number of Genes
     * in Population Annotated with the Term), otherwise will be six
     * (GO Term ID, GO Term Name, Number of Genes in Population
     * Annotated with the Term, Number of Genes in Set Annotated
     * with the Term, Hypergeometric P-Value, Bonferroni Corrected
     * Hypergeometric P-Value).
     *
     * @return String[] the actual columns names for this table
     */
    public String[] getActualColumnNames() {
        if (this.getModel().getColumnCount() == 3) {
            String[] names = {columnNames[0], columnNames[1],
                             columnNames[2]};
            return names;
        }
        return columnNames;
    }

    /**
     * Returns the actual complete, not abbreviated column names
     * for this table. If this table displays only terms information
     * (no p-values), columns names will be three (GO Term ID, GO Term
     * Name, Number of Genes in Population Annotated with the Term),
     * otherwise will be six (GO Term ID, GO Term Name, Number of
     * Genes in Population Annotated with the Term, Number of Genes in
     * Set Annotated with the Term, Hypergeometric P-Value, Bonferroni
     * Corrected Hypergeometric P-Value).
     *
     * @return String[] the actual columns names for this table
     */
    public String[] getActualCompleteColumnNames() {
        if (this.getModel().getColumnCount() == 3) {
            String[] names = {completeColumnNames[0], completeColumnNames[1],
                             completeColumnNames[2]};
            return names;
        }
        return completeColumnNames;
    }

    /**
     * Sets a new color to mark highly significant terms.
     *
     * @param newHighlySignificantColor <code>Color</code> to mark
     * highly significant terms
     */
    public void getHighlySignificantColor(Color newHighlySignificantColor) {
        highlySignificantColor = newHighlySignificantColor;
    }

    /**
     * Sets a new color to mark significant terms.
     *
     * @param newSignificantColor <code>Color</code> to mark
     * significant terms
     */
    public void setSignificantColor(Color newSignificantColor) {
        significantColor = newSignificantColor;
    }

    /**
     * Sets a new color to mark even rows that are not selected.
     *
     * @param newEvenRowsColor <code>Color</code> to mark even
     * rows that are not selected
     */
    public void setEvenRowsColor(Color newEvenRowsColor) {
        evenRowsColor = newEvenRowsColor;
    }

    /**
     * Sets a new color to mark odd rows that are not selected.
     *
     * @param newOddRowsColor <code>Color</code> to mark odd
     * rows that are not selected
     */
    public void setOddRowsColor(Color newOddRowsColor) {
        oddRowsColor = newOddRowsColor;
    }

    /**
     * Handles the event fired when the user clicks a row of this table.
     * Shows a popup window with the annotated genes for the clicked term.
     *
     * @param e <code>MouseEvent</code> the event fired when the user
     *        clicks a row of this table
     */
    public void this_mouseClicked(MouseEvent e) {
        if (goFrontEnd == null) {
            return;
        }

        Point p = e.getPoint();

        int viewRowIndex = this.rowAtPoint(p);

        int goTermID = Integer.parseInt(((String)this.getValueAt(viewRowIndex,
                0)).substring(3));

/*        String annotatedGenesNames = "";
        long time_init = System.currentTimeMillis();
        int count = 0;
        for (int i = 0; i < genesNames.length; i++) {
            if (goFrontEnd.termQueryFull(goTermID, genesNames[i])) {
                annotatedGenesNames += "\n" + genesNames[i];
                count++;
            }
        }
        System.out.println("OLD ALGORITHM: " + (System.currentTimeMillis()-time_init));
 */

        int[] numberOfAnnotatedGenes = new int[]{0};
        String annotatedGenes =
            goFrontEnd.queryGOTermIDForGenes(goTermID, genesNames, numberOfAnnotatedGenes);

        if (numberOfAnnotatedGenes[0] == 0) {
            JOptionPane.showMessageDialog(parent,
                                          "No genes annotated with term " +
                                          this.getValueAt(viewRowIndex, 0),
                                          "No genes annotated",
                                          JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        JTextArea textArea = new JTextArea("GO Term ID: " +
                                           this.getValueAt(viewRowIndex, 0) +
                                           "\nAnnotated genes: " +
                                           numberOfAnnotatedGenes[0] +
                                           "\n" + annotatedGenes);
        textArea.setEditable(false);
        JScrollPane scroll = new JScrollPane(textArea);
        scroll.setPreferredSize(new Dimension(60, 250));
        scroll.getVerticalScrollBar().
                setUnitIncrement(BiggestsFrame.SCROLL_BAR_INCREMENT/2);
        JOptionPane.showMessageDialog(parent, scroll, "Annotated Genes",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     *
     * <p>Title: <code>NumberCellRenderer</code>: A renderer for cells which
     *           contain numbers.</p>
     *
     * <p>Description: </p>
     *
     * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
     *                 This program is free software; you can redistribute
     *                 it and/or modify it under the terms of the GNU General
     *                 Public License as published by the Free Software
     *                 Foundation; either version 2 of the License, or
     *                 any later version.
     *
     *                 This program is distributed in the hope that it will
     *                 be useful, but WITHOUT ANY WARRANTY; without even the
     *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
     *                 PARTICULAR PURPOSE.  See the
     *                 <a href="http://www.gnu.org/copyleft/gpl.html">
     *                 GNU General Public License</a> for more details.
     * </p>
     *
     * @author Joana Gonalves
     */
    protected static class NumberCellRenderer extends DefaultTableCellRenderer.
            UIResource {
        /**
         * Constructs a new <code>NumberCellRenderer</code> for a cell.
         *
         * @see javax.swing.table.DefaultTableCellRenderer#DefaultTableCellRenderer()
         */
        public NumberCellRenderer() {
            super();
            setHorizontalAlignment(JLabel.RIGHT);
        }
    }


    /**
     * <p>Title: <code>FloatCellRenderer</code>: A renderer for cells which
     *           contain float numbers.</p>
     *
     * <p>Description: </p>
     *
     * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
     *                 This program is free software; you can redistribute
     *                 it and/or modify it under the terms of the GNU General
     *                 Public License as published by the Free Software
     *                 Foundation; either version 2 of the License, or
     *                 any later version.
     *
     *                 This program is distributed in the hope that it will
     *                 be useful, but WITHOUT ANY WARRANTY; without even the
     *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
     *                 PARTICULAR PURPOSE.  See the
     *                 <a href="http://www.gnu.org/copyleft/gpl.html">
     *                 GNU General Public License</a> for more details.
     * </p>
     *
     * @author Joana Gonalves
     */
    protected static class FloatCellRenderer extends NumberCellRenderer {
        /**
         * The formatter which is used to format the float value.
         */
        NumberFormat formatter;

        /**
         * Constructs a new <code>FloatCellRenderer</code>.
         */
        public FloatCellRenderer() {
            super();
        }

        /**
         * Overrides <code>DefaultTableCellRenderer</code>'s
         * <code>setValue</code> method. Formats the value of the cell to display.
         *
         * @param value the <code>Object</code> containing the float value
         */
        public void setValue(Object value) {
            if (formatter == null) {
                formatter = NumberFormat.getInstance(Locale.ENGLISH);
                formatter.setMaximumFractionDigits(8);
                formatter.setMinimumFractionDigits(8);
            }
            setText((value == null) ? "" : formatter.format(value));
        }
    }
}


/**
 *
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 2 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/copyleft/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonalves
 */
class TableFunctionIndividual_this_mouseAdapter extends MouseAdapter {
    private TableFunctionIndividual adaptee;
    TableFunctionIndividual_this_mouseAdapter(TableFunctionIndividual adaptee) {
        this.adaptee = adaptee;
    }

    public void mouseClicked(MouseEvent e) {
        adaptee.this_mouseClicked(e);
    }
}
