package biggests.tree;

import java.io.*;
import java.util.*;

import javax.swing.text.*;
import javax.swing.tree.*;

import biggests.utils.*;
import smadeira.biclustering.*;

/**
 * <p>Title: Node Information Object</p>
 *
 * <p>Description: Contains information about the content of a
 *                 specific node in the dataset tree.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonalves
 */

public class NodeInfo implements Serializable {
    public static final long serialVersionUID = 7553487355938947549L;

    /**
     * The original node object type.
     */
    public static final int ORIGINAL = 0;

    /**
     * The preprocessed node object type.
     */
    public static final int PREPROCESSED = 1;

    /**
     * The discretized node object type.
     */
    public static final int DISCRETIZED = 2;

    /**
     * The bicluster group node object type.
     */
    public static final int BICLUSTER_GROUP = 3;

    /**
     * The single bicluster node object type.
     */
    public static final int BICLUSTER_SINGLE = 5;

    /**
     * The post-processed group node object type.
     */
    public static final int POSTPROCESSED_GROUP = 4;

    /**
     * The post-processed single node object type.
     */
    public static final int POSTPROCESSED_SINGLE = 6;

    /**
     * The float values matrix data type.
     */
    public static final boolean VALUES = false;

    /**
     * The symbol values matrix data type.
     */
    public static final boolean SYMBOLS = true;

    /**
     * The name to be displayed with the id, as the tree node name.
     */
    private String name;

    /**
     * Id number to be displayed with the name, as the tree node name.
     */
    private int id;

    /**
     * Id number corresponding to the original bicluster id before
     * post-processing operation.
     */
    private int fakeID;

    /**
     * Node object type.<br/>
     * One of the following:<br/>
     *    <code>NodeInfo.ORIGINAL</code><br/>
     *    <code>NodeInfo.PREPROCESSED</code><br/>
     *    <code>NodeInfo.DISCRETIZED</code><br/>
     *    <code>NodeInfo.BICLUSTER_GROUP</code><br/>
     *    <code>NodeInfo.BICLUSTER_SINGLE</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_GROUP</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_SINGLE</code><br/>
     */
    private int nodeObjectType;

    /**
     * Type of data used in the matrix of the node object.<br/>
     * One of the two:<br/>
     *     <code>VALUES</code><br/>
     *     <code>SYMBOLS</code><br/>
     */
    private boolean dataType;

    /**
     * Type of genes' names.<br/>
     * <code>true</code> - genes' names are gene IDs (gene ontology names)<br/>
     * <code>false</code> - genes' names are probeset IDs (IDs used in the mRNA measurements<br/>
     */
    private boolean geneID;

    /**
     * The node object.
     */
    private NodeObjectInterface nodeObject;

    /**
     * Parent node information object.
     */
    private NodeInfo parentNodeInfo;

    /**
     * Information strings.
     */
    private ArrayList<String> infoStrings;

    /**
     * Information styles;
     */
    private ArrayList<String> infoStyles;

    /**
     * Constructs an object of type <code>NodeInfo</code> (with all the information about
     * a certain node, including a pointer for the node object itself.
     *
     * @param nodeObject NodeObjectInterface the node object
     * @param nodeObjectType int the type of the node object:<br/>
     *    <code>NodeInfo.ORIGINAL</code><br/>
     *    <code>NodeInfo.PREPROCESSED</code><br/>
     *    <code>NodeInfo.DISCRETIZED</code><br/>
     *    <code>NodeInfo.BICLUSTER_GROUP</code><br/>
     *    <code>NodeInfo.BICLUSTER_SINGLE</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_GROUP</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_SINGLE</code><br/>
     * @param selectionPath TreePath the selected path
     * @param tree DatasetTree the dataset tree
     * @param parent the parent <code>NodeInfo</code> of this <code>NodeInfo</code> object's
     * parent group (only for post-processed biclusters <code>NodeInfo</code> objects)
     * @param infoStrings the information <code>String[]</code>
     * @param infoStyles the styles <code>String[]</code>
     * @param index <code>int</code> the index of the bicluster in the array
     *              of biclusters (it applies only to post-processed biclusters
     * @throws InvalidNodeObjectTypeException if the type of the node object is not valid
     */
    public NodeInfo(NodeObjectInterface nodeObject
                    , int nodeObjectType
                    , TreePath selectionPath
                    , DatasetTree tree
                    , NodeInfo parent
                    , ArrayList<String> infoStrings
                    , ArrayList<String> infoStyles
                    , int index) throws InvalidNodeObjectTypeException {
        this.fakeID = -1;
        parentNodeInfo = parent;
        if (nodeObject instanceof ExpressionMatrix) {
            this.geneID = ((ExpressionMatrix) nodeObject).getGeneIdentifier().
                          equals(
                                  "Gene ID");
        } else {
            if (parentNodeInfo != null) {
                this.geneID = parentNodeInfo.geneID;
            }
        }
        this.infoStrings = infoStrings;
        this.infoStyles = infoStyles;

        /**
         * give a name and id according to the type of the node object
         */
        switch (nodeObjectType) {
        case ORIGINAL:
            if (!(nodeObject instanceof ExpressionMatrix)) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match the type of the object node"));
            }

            name = "Original";
            id = 0;
            dataType = VALUES;
            break;
        case PREPROCESSED:
            if (!(nodeObject instanceof PreProcessedExpressionMatrix)) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match the type of the object node"));
            }

            name = "Preprocessed";
            id = tree.generateNewNodeInfoId(nodeObjectType, selectionPath, true);
            dataType = VALUES;
            break;
        case DISCRETIZED:
            if (!(nodeObject instanceof DiscretizedExpressionMatrix)) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match the type of the object node"));
            }

            name = "Discretized";
            id = tree.generateNewNodeInfoId(nodeObjectType, selectionPath, true);
            dataType = SYMBOLS;
            break;
        case BICLUSTER_GROUP:
            if (!((nodeObject instanceof Biclustering) ||
                  (nodeObject instanceof CC_TSB_Biclustering) ||
                  (nodeObject instanceof CCC_Biclustering) ||
                  (nodeObject instanceof E_CCC_Biclustering))) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match the type of the object node"));
            }

            name = "Biclusters Group";
            id = tree.generateNewNodeInfoId(nodeObjectType, selectionPath, true);

            if ((nodeObject instanceof CCC_Biclustering) ||
                (nodeObject instanceof E_CCC_Biclustering)) {
                dataType = SYMBOLS;
            } else
            if ((nodeObject instanceof Biclustering) ||
                (nodeObject instanceof CC_TSB_Biclustering)) {
                dataType = VALUES;
            }
            break;
        case POSTPROCESSED_GROUP:
            if (!((nodeObject instanceof Biclustering) ||
                  (nodeObject instanceof CC_TSB_Biclustering) ||
                  (nodeObject instanceof CCC_Biclustering) ||
                  (nodeObject instanceof E_CCC_Biclustering))) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match with the type of the object node"));
            }

            name = "Post-Processed Group";
            id = tree.generateNewNodeInfoId(nodeObjectType, selectionPath, true);

            if ((nodeObject instanceof CCC_Biclustering) ||
                (nodeObject instanceof E_CCC_Biclustering)) {
                dataType = SYMBOLS;
            } else
            if ((nodeObject instanceof Biclustering) ||
                (nodeObject instanceof CC_TSB_Biclustering)) {
                dataType = VALUES;
            }
            break;
        case BICLUSTER_SINGLE:
            if (!((nodeObject instanceof Bicluster) ||
                  (nodeObject instanceof CCC_Bicluster) ||
                  (nodeObject instanceof E_CCC_Bicluster))) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match with the type of the object node"));
            }

            name = "Bicluster";
            id = ((Bicluster) nodeObject).getID();

            if ((nodeObject instanceof CCC_Bicluster) ||
                (nodeObject instanceof E_CCC_Bicluster)) {
                dataType = SYMBOLS;
            } else
            if (nodeObject instanceof Bicluster) {
                dataType = VALUES;
            }
            break;
        case POSTPROCESSED_SINGLE:
            if (!((nodeObject instanceof Bicluster) ||
                  (nodeObject instanceof CCC_Bicluster) ||
                  (nodeObject instanceof E_CCC_Bicluster))) {
                throw (new InvalidNodeObjectTypeException(
                        "nodeObjectType doesn't match with the type of the object node"));
            }

            name = "Post";
            id = ((Bicluster) nodeObject).getID();
            fakeID = index + 1;

            if ((nodeObject instanceof CCC_Bicluster) ||
                (nodeObject instanceof E_CCC_Bicluster)) {
                dataType = SYMBOLS;
            } else
            if (nodeObject instanceof Bicluster) {
                dataType = VALUES;
            }
            break;
        default:

            /**
             * The value for nodeObjectType is not valid, because it is not one
             * of the following:
             *      NodeInfo.ORIGINAL
             *      NodeInfo.PREPROCESSED
             *      NodeInfo.DISCRETIZED
             *      NodeInfo.BICLUSTER_GROUP
             *      NodeInfo.BICLUSTER_SINGLE
             *      NodeInfo.POSTPROCESSED_GROUP
             *      NodeInfo.POSTPROCESSED_SINGLE
             */
            throw (new InvalidNodeObjectTypeException(
                    "Only original, preprocessed, discretized, bicluster and post-processed groups, single biclusters and post-processed biclusters are allowed"));
        }

        this.nodeObject = nodeObject;
        this.nodeObjectType = nodeObjectType;
    }

    /**
     * Constructs an object of type <code>NodeInfo</code> (with all the information about
     * a certain node, including a pointer for the node object itself.
     *
     * @param nodeObject NodeObjectInterface the node object
     * @param nodeObjectType int the type of the node object:<br/>
     *    <code>NodeInfo.ORIGINAL</code><br/>
     *    <code>NodeInfo.PREPROCESSED</code><br/>
     *    <code>NodeInfo.DISCRETIZED</code><br/>
     *    <code>NodeInfo.BICLUSTER_GROUP</code><br/>
     *    <code>NodeInfo.BICLUSTER_SINGLE</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_GROUP</code><br/>
     *    <code>NodeInfo.POSTPROCESSED_SINGLE</code><br/>
     * @param selectionPath TreePath the selected path
     * @param tree DatasetTree the dataset tree
     * @param parent the parent <code>NodeInfo</code> of this <code>NodeInfo</code> object
     * @param infoStrings the information <code>String[]</code>
     * @param infoStyles the styles <code>String[]</code>
     * @throws InvalidNodeObjectTypeException if the type of the node object is not valid
     */
    public NodeInfo(NodeObjectInterface nodeObject
                    , int nodeObjectType
                    , TreePath selectionPath
                    , DatasetTree tree
                    , NodeInfo parent
                    , ArrayList<String> infoStrings
                    , ArrayList<String> infoStyles) throws
            InvalidNodeObjectTypeException {
        this(nodeObject, nodeObjectType, selectionPath, tree, parent,
             infoStrings, infoStyles, -1);
    }

    /**
     * Returns the name of this node information object.
     *
     * @return the <code>String</code> name of this node information object
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the number id of the <code>NodeInfo<code/> object.
     *
     * @return int the number id
     */
    public int getId() {
        return id;
    }

    /**
     * Returns the node object type.<br/>
     * One of the following:<br/>
     *      <code>NodeInfo.ORIGINAL</code><br/>
     *      <code>NodeInfo.PREPROCESSED</code><br/>
     *      <code>NodeInfo.DISCRETIZED</code><br/>
     *      <code>NodeInfo.BICLUSTER_GROUP</code><br/>
     *      <code>NodeInfo.BICLUSTER_SINGLE</code><br/>
     *      <code>NodeInfo.POSTPROCESSED_GROUP</code><br/>
     *      <code>NodeInfo.POSTPROCESSED_SINGLE</code><br/>
     *
     * @return int the node object type
     */
    public int getNodeObjectType() {
        return nodeObjectType;
    }

    /**
     * Returns the node object data type value.<br/>
     * One of the following:<br/>
     *      <code>NodeInfo.VALUES</code><br/>
     *      <code>NodeInfo.SYMBOLS</code><br/>
     *
     * @return boolean the data type of the node object
     */
    public boolean getDataType() {
        return dataType;
    }

    /**
     * Returns genes' names type.
     * <code>true</code> - genes' names are gene IDs (gene ontology names)<br/>
     * <code>false</code> - genes' names are probeset IDs (IDs used in the mRNA measurements<br/>
     *
     * @return boolean the type of genes' names
     */
    public boolean isGeneID() {
        return geneID;
    }

    /**
     * Returns the node object associated with the current node information object.
     *
     * @return NodeObjectInterface the node object
     */
    public NodeObjectInterface getNodeObject() {
        return nodeObject;
    }

    /**
     * Returns parent node information object.
     *
     * @return NodeInfo parent node information object
     */
    public NodeInfo getParentNodeInfo() {
        return parentNodeInfo;
    }

    /**
     * Returns information strings.
     *
     * @return String[] information strings
     */
    public ArrayList<String> getInfoStrings() {
        if (nodeObjectType == BICLUSTER_SINGLE ||
            nodeObjectType == POSTPROCESSED_SINGLE) {
            Bicluster bic = (Bicluster) nodeObject;
            ArrayList<String> infoStrings = new ArrayList<String>();

/*            float pValue_Threshold = bic.getBiclusterSource().
                                     getGO_pValues_userThreshold();
 */
            String organism = null;
            IMatrix matrix = (IMatrix) bic.getBiclusterSource().getMatrix();
            if (matrix instanceof AbstractExpressionMatrix) {
                organism = ((AbstractExpressionMatrix) matrix).getOrganism();
            } else {
                organism = ((DiscretizedExpressionMatrix) matrix).
                           getOriginalExpressionMatrix().getOrganism();
            }

            String datatype;
            if (dataType == NodeInfo.SYMBOLS) {
                datatype = "discrete symbols";
            } else {
                datatype = "floating point values";
            }

            infoStrings.add("Number of genes: ");
            infoStrings.add(bic.getNumberOfGenes() + "\n");
            infoStrings.add("Number of time points: ");
            infoStrings.add(bic.getNumberOfConditions() + "\n");
            infoStrings.add("Size: ");
            infoStrings.add(bic.computeSIZE() + "\n\n");

            infoStrings.add("Datatype: ");
            infoStrings.add(datatype + "\n");
            infoStrings.add("Organism: ");
            infoStrings.add(organism + "\n\n");

/*            infoStrings.add("Average column variance: ");
            infoStrings.add(bic.computeACV() + "\n");
            infoStrings.add("Average row variance: ");
            infoStrings.add(bic.computeARV() + "\n");
            infoStrings.add("Mean squared residue: ");
            infoStrings.add(bic.computeMSR() + "\n");
            infoStrings.add("Variance: ");
            infoStrings.add(bic.computeVAR() + "\n\n");

            if (dataType == NodeInfo.SYMBOLS) {
                infoStrings.add("Pattern p-value: ");
                infoStrings.add(((CCC_Bicluster)bic).
                                getBiclusterPattern_pValue(1) + "\n");
                infoStrings.add("Pattern w/ columns p-value: ");
                infoStrings.add(((CCC_Bicluster)bic).
                                getBiclusterPatternWithColumns_pValue(1) + "\n\n");
            }
 */

/*            if (bic.getGeneOntologyTerms() != null) {
                infoStrings.add("Function analysis\n");
                infoStrings.add("   best p-value: ");
                infoStrings.add(bic.getBest_GO_pValue() + "\n");
                infoStrings.add("   best Bonferroni corrected p-value: ");
                infoStrings.add(bic.
                                       getBest_GO_BonferroniCorrected_pValue() +
                                       "\n");
                infoStrings.add("   number of significant functions: ");
                infoStrings.add(bic.
                                       getSignificant_GO_BonferroniCorrected_pValues_UserThreshold(
                        pValue_Threshold) + "\n");
            }
 */

            return infoStrings;
        }
        return infoStrings;
    }

    /**
     * Returns information styles.
     *
     * @return ArrayList<String> information styles
     */
    public ArrayList<String> getInfoStyles() {
        if (nodeObjectType == BICLUSTER_SINGLE ||
            nodeObjectType == POSTPROCESSED_SINGLE) {
            Bicluster bic = (Bicluster) nodeObject;
            ArrayList<String> infoStyles = new ArrayList<String>();

            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");

            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");

            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");
            infoStyles.add("bold");
            infoStyles.add("regular");

            if (dataType == NodeInfo.SYMBOLS) {
                infoStyles.add("bold");
                infoStyles.add("regular");
                infoStyles.add("bold");
                infoStyles.add("regular");
            }

/*            if (bic.getGeneOntologyTerms() != null) {
                infoStyles.add("bold");
                infoStyles.add("regular");
                infoStyles.add("regular");
                infoStyles.add("regular");
                infoStyles.add("regular");
                infoStyles.add("regular");
                infoStyles.add("regular");
            }
 */

            return infoStyles;
        }
        return infoStyles;
    }

    /**
     * Returns the ID of the post-processed bicluster.
     *
     * @return int the ID of the post-processed bicluster
     */
    public int getFakeID() {
        return fakeID;
    }

    /**
     * Sets information strings.
     *
     * @param strings information <code>ArrayList<String></code>
     */
    public void setInfoStrings(ArrayList<String> strings) {
        infoStrings = strings;
    }

    /**
     * Sets information styles.
     *
     * @param styles information <code>ArrayList<String></code> styles
     */
    public void setInfoStyles(ArrayList<String> styles) {
        infoStyles = styles;
    }

    /**
     * Returns a styled document containing information about this node.
     *
     * @return StyledDocument containing information about this node
     * @throws BadLocationException if strings or styles could not be found
     */
    public StyledDocument getInfo() throws BadLocationException {
        StyledDocument document = new DefaultStyledDocument();
        addStylesToDocument(document);

        if (nodeObjectType != NodeInfo.POSTPROCESSED_SINGLE) {
            document.insertString(document.getLength(),
                                  this.toString() + "\n\n",
                                  document.getStyle("bold"));
        }
        else {
            document.insertString(document.getLength(),
                                  "Post-Processed Bicluster " + fakeID + "   (" +
                                  id + ")\n\n",
                                  document.getStyle("bold"));
            document.insertString(document.getLength(),
                                  "Post-processed bicluster id: ",
                                  document.getStyle("bold"));
            document.insertString(document.getLength(),
                                  fakeID + "\n",
                                  document.getStyle("regular"));
            document.insertString(document.getLength(),
                                  "Corresponding parent id: ",
                                  document.getStyle("bold"));
            document.insertString(document.getLength(),
                                  id + "\n\n",
                                  document.getStyle("regular"));
            document.insertString(document.getLength(),
                                  "Parent group node:\n", document.getStyle("bold"));
            document.insertString(document.getLength(),
                                  "   " + parentNodeInfo.toString() + "\n",
                                  document.getStyle("regular"));
        }

        if (parentNodeInfo != null &&
            nodeObjectType != NodeInfo.POSTPROCESSED_SINGLE) {
            document.insertString(document.getLength(),
                                  "Parent node: ", document.getStyle("bold"));
            document.insertString(document.getLength(),
                                  parentNodeInfo.toString() + "\n",
                                  document.getStyle("regular"));
        }
        ArrayList<String> infoStrings = this.getInfoStrings();
        ArrayList<String> infoStyles = this.getInfoStyles();
        if (infoStrings != null && infoStyles != null) {
            for (int i = 0; i < infoStrings.size(); i++) {
                document.insertString(document.getLength(),
                                      infoStrings.get(i),
                                      document.getStyle(infoStyles.get(i)));
            }
        }
        return document;
    }

    /**
     * Finds a specific information string.
     *
     * @param s the information <code>String</code> to look for
     * @return int the index of the information string in the array
     *         (-1 if it couldn't find it)
     */
    public int findInfoString(String s) {
        for (int i = 0; i < s.length(); i++) {
            if (infoStrings.get(i).equals(s)) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Adds a set of common styles to <code>StyledDocument</code> <code>doc</code>
     *
     * @param doc the <code>StyledDocument</code> to add styles to
     */
    protected void addStylesToDocument(StyledDocument doc) {
        //Initialize some styles.
        Style def = StyleContext.getDefaultStyleContext().
                    getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = doc.addStyle("regular", def);
        StyleConstants.setFontFamily(def, "SansSerif");

        Style s = doc.addStyle("italic", regular);
        StyleConstants.setItalic(s, true);

        s = doc.addStyle("bold", regular);
        StyleConstants.setBold(s, true);

        s = doc.addStyle("small", regular);
        StyleConstants.setFontSize(s, 10);

        s = doc.addStyle("large", regular);
        StyleConstants.setFontSize(s, 16);
    }

    /**
     * Overrides the default <code>toString</code> method. Returns a formatted
     * string with the name to display in the corresponding dataset tree node.
     *
     * @return String the name of the node object, which is displayed in the datasets tree
     */
    public String toString() {
        if (fakeID != -1) {
            return name + " " + fakeID + "        (" + id + ")";
        } else {
            return name + " " + id;
        }
    }
}
